import json
import boto3
from botocore.exceptions import ClientError


class CredStore:
    def __init__(self, config, region):
        self.config = config
        self.region = region

    def get_ssm_parameter(self, client, parameter_name):
        """Retrieve a parameter from AWS SSM Parameter Store."""
        try:
            response = client.get_parameter(Name=parameter_name, WithDecryption=True)
            return response['Parameter']['Value']
        except ClientError as e:
            raise Exception(f"Error retrieving SSM parameter ({parameter_name}): {e}") from e

    def get_secret(self, client, secret_name):
        """Retrieve a secret from AWS Secrets Manager."""
        try:
            response = client.get_secret_value(SecretId=secret_name)
            if 'SecretString' in response:
                return json.loads(response['SecretString'])
            raise Exception(f"SecretString not found in the response for secret ({secret_name})")
        except ClientError as e:
            error_code = e.response['Error']['Code']
            if error_code == 'ResourceNotFoundException':
                raise Exception(f"The requested secret ({secret_name}) was not found") from e
            if error_code == 'InvalidRequestException':
                raise Exception(f"The request was invalid due to: {e}") from e
            if error_code == 'InvalidParameterException':
                raise Exception(f"The request had invalid params: {e}") from e
            raise Exception(f"Error retrieving Secrets Manager secret ({secret_name}): {e}") from e

    def validate_secret_keys(self, secret, client_id_key, client_secret_key):
        """Validate the presence of required keys in the secret."""
        falcon_client_id = secret.get(client_id_key)
        falcon_client_secret = secret.get(client_secret_key)

        if falcon_client_id is None:
            raise Exception(f"The client ID key ({client_id_key}) does not exist or is None in the retrieved secret.")
        if falcon_client_secret is None:
            raise Exception(f"The client secret key ({client_secret_key}) does not exist or is None in the retrieved secret.")

        return falcon_client_id, falcon_client_secret

    def load_credentials(self, store):
        """Load credentials based on the specified store."""
        if store == 'ssm':
            ssm_client = boto3.client('ssm', region_name=self.region)
            falcon_client_id = self.get_ssm_parameter(
                ssm_client,
                self.config.get('ssm', 'ssm_client_id')
            )
            falcon_client_secret = self.get_ssm_parameter(
                ssm_client,
                self.config.get('ssm', 'ssm_client_secret')
            )
        elif store == 'secrets_manager':
            secrets_client = boto3.client('secretsmanager', region_name=self.region)
            secret = self.get_secret(
                secrets_client,
                self.config.get('secrets_manager', 'secrets_manager_secret_name')
            )
            falcon_client_id, falcon_client_secret = self.validate_secret_keys(
                secret,
                self.config.get('secrets_manager', 'secrets_manager_client_id_key'),
                self.config.get('secrets_manager', 'secrets_manager_client_secret_key')
            )
        else:
            raise ValueError("Invalid credentials store specified.")

        return falcon_client_id, falcon_client_secret



from ...log import log


class Runtime():
    RELEVANT_EVENT_TYPES = "ALL"

    def __init__(self):
        log.info("GENERIC Backend is enabled.")

    def is_relevant(self, falcon_event):
        return True

    def process(self, falcon_event):
        # Used to display falcon_evnts in the console
        log.info(falcon_event.original_event)


__all__ = ['Runtime']



from datetime import datetime
import logging
import traceback
import boto3
from botocore.exceptions import ClientError
from ...config import config
from ...log import log


class Submitter():
    def __init__(self, event):
        self.event = event
        self.region = config.get('aws', 'region')
        self.confirm_instance = config.getboolean('aws', 'confirm_instance')

    def find_instance(self, instance_id, mac_address):
        # Instance IDs are unique to the region, not the account, so we have to check them all
        report_region = self.region
        ec2instance = None
        ec2_client = boto3.client("ec2", region_name=report_region)
        regions = [region["RegionName"] for region in ec2_client.describe_regions()["Regions"]]
        for region in regions:
            ec2 = boto3.resource("ec2", region_name=region)
            try:
                ec2instance = ec2.Instance(instance_id)
                found = False
                # Confirm the mac address matches
                if ec2instance.network_interfaces:
                    for iface in ec2instance.network_interfaces:
                        det_mac = mac_address.lower().replace(":", "").replace("-", "")
                        ins_mac = iface.mac_address.lower().replace(":", "").replace("-", "")
                        if det_mac == ins_mac:
                            found = True
                if found:  # pylint: disable=R1723
                    return region, ec2instance
            except ClientError:
                continue
            except Exception:  # pylint: disable=W0703
                trace = traceback.format_exc()
                log.exception(str(trace))
                continue

        return report_region, ec2instance

    @staticmethod
    def send_to_securityhub(manifest, region):
        client = boto3.client('securityhub', region_name=region)
        check_response = {}
        found = False
        try:
            check_response = client.get_findings(Filters={'Id': [{'Value': manifest["Id"], 'Comparison': 'EQUALS'}]})
            for _ in check_response["Findings"]:
                found = True
        except ClientError:
            pass

        import_response = False
        if not found:
            try:
                import_response = client.batch_import_findings(Findings=[manifest])
            except ClientError as err:
                # Boto3 issue communicating with SH, throw the error in the log
                log.exception(str(err))

        return import_response

    def submit(self):
        log.info("Processing detection: %s", self.event.detect_description)
        det_region = self.region
        send = False
        if self.confirm_instance:
            try:
                if self.event.instance_id:
                    det_region, instance = self.find_instance(self.event.instance_id, self.event.device_details["mac_address"])
                    if instance is None:
                        log.warning("Instance %s with MAC address %s not found in regions searched. Alert not processed.",
                                    self.event.instance_id, self.event.device_details["mac_address"])
                        return
                    try:
                        if instance.network_interfaces:
                            for _ in instance.network_interfaces:
                                # Only send alerts for instances we can find
                                send = True

                    except ClientError:
                        # Not our instance
                        i_id = self.event.instance_id
                        mac = self.event.device_details["mac_address"]
                        log.info("Instance %s with MAC address %s not found in regions searched. Alert not processed.", i_id, mac)
            except AttributeError:
                # Instance ID was not provided by the detection
                log.info("Instance ID not provided by detection. Alert not processed.")
        else:
            # If we're not confirming the instance, we can just send the alert
            send = True

        if send:
            sh_payload = self.create_payload(det_region)
            response = self.send_to_securityhub(sh_payload, det_region)
            if not response:
                if log.level <= logging.DEBUG:
                    log.debug("Detection already submitted to Security Hub. Alert not processed. Payload info: %s", sh_payload)
                else:
                    log.info("Detection already submitted to Security Hub. Alert not processed.")
            else:
                if response["SuccessCount"] > 0:
                    if log.level <= logging.DEBUG:
                        log.debug("Detection submitted to Security Hub. (Request ID: %s). Payload info: %s",
                                  response['ResponseMetadata']['RequestId'], sh_payload)
                    else:
                        submit_msg = f"Detection submitted to Security Hub. (Request ID: {response['ResponseMetadata']['RequestId']})"
                        log.info(submit_msg)

    def create_payload(self, instance_region):
        region = self.region
        try:
            account_id = boto3.client("sts", region_name=region).get_caller_identity().get('Account')
        except KeyError:
            # Failed to get endpoint_resolver the first time, try it again
            account_id = boto3.client("sts", region_name=region).get_caller_identity().get('Account')
        severity_original = self.event.severity
        severity_label = severity_original.upper()
        if "gov" in region:
            ARN = "arn:aws-us-gov:securityhub:{}:358431324613:product/crowdstrike/crowdstrike-falcon".format(region)
        else:
            ARN = "arn:aws:securityhub:{}:517716713836:product/crowdstrike/crowdstrike-falcon".format(region)
        payload = {
            "SchemaVersion": "2018-10-08",
            "ProductArn": f"{ARN}",
            "AwsAccountId": account_id,
            "SourceUrl": self.event.falcon_link,
            "GeneratorId": "Falcon Host",
            "CreatedAt": datetime.utcfromtimestamp(float(self.event.event_create_time) / 1000.).isoformat() + 'Z',
            "UpdatedAt": ((datetime.utcfromtimestamp(datetime.timestamp(datetime.now()))).isoformat() + 'Z'),
            "RecordState": "ACTIVE",
            "Severity": {"Label": severity_label, "Original": severity_original},
            "ProductFields": {"crowdstrike/crowdstrike-falcon/cid": self.event.cid},
        }

        # Instance ID based detail
        try:
            payload["Id"] = f"{self.event.instance_id}{self.event.event_id}"
            payload["Title"] = "Falcon Alert. Instance: %s" % self.event.instance_id
            payload["Resources"] = [{"Type": "AwsEc2Instance", "Id": self.event.instance_id, "Region": instance_region}]
        except AttributeError:
            payload["Id"] = f"UnknownInstanceID:{self.event.event_id}"
            payload["Title"] = "Falcon Alert"
            payload["Resources"] = [{"Type": "Other",
                                     "Id": f"UnknownInstanceId:{self.event.event_id}",
                                     "Region": region
                                     }]
        # Description
        aws_id = ""
        if self.event.cloud_provider_account_id:
            aws_id = f"| AWS Account for alerting instance: {self.event.cloud_provider_account_id}"
        payload["Description"] = f"{self.event.detect_description} {aws_id}"

        # TTPs
        try:
            payload["Types"] = ["Namespace: TTPs",
                                "Category: %s" % self.event.original_event["event"]["Tactic"],
                                "Classifier: %s" % self.event.original_event["event"]["Technique"]
                                ]
        except KeyError:
            payload.pop("Types", None)

        # Running process detail
        try:
            payload["Process"] = {}
            payload["Process"]["Name"] = self.event.original_event["event"]["FileName"]
            payload["Process"]["Path"] = self.event.original_event["event"]["FilePath"]
        except KeyError:
            payload.pop("Process", None)

        # Network detail
        try:
            payload['Network'] = self.network_payload()
        except KeyError:
            pass

        return payload

    def network_payload(self):
        net = {}
        net['Direction'] = \
            "IN" if self.event.original_event['event']['NetworkAccesses'][0]['ConnectionDirection'] == 0 else 'OUT'
        net['Protocol'] = self.event.original_event['event']['NetworkAccesses'][0]['Protocol']
        net['SourceIpV4'] = self.event.original_event['event']['NetworkAccesses'][0]['LocalAddress']
        net['SourcePort'] = self.event.original_event['event']['NetworkAccesses'][0]['LocalPort']
        net['DestinationIpV4'] = self.event.original_event['event']['NetworkAccesses'][0]['RemoteAddress']
        net['DestinationPort'] = self.event.original_event['event']['NetworkAccesses'][0]['RemotePort']
        return net


class Runtime():
    RELEVANT_EVENT_TYPES = ['EppDetectionSummaryEvent']

    def __init__(self):
        log.info("AWS Backend is enabled.")

    def is_relevant(self, falcon_event):
        if falcon_event.cloud_provider is not None:
            return falcon_event.cloud_provider[:3].upper() == 'AWS'
        return False

    def process(self, falcon_event):
        Submitter(falcon_event).submit()


__all__ = ['Runtime']



import json
import re
import datetime
from ..config import config
from ..log import log


class Event(dict):
    def __init__(self, event_string, feed_id):
        event = json.loads(event_string.decode('utf-8'))
        self.feed_id = feed_id
        super().__init__(event)

    def __eq__(self, other):
        if not isinstance(other, Event):
            return False
        return super().__eq__(self, other) and self.feed_id == other.feed_id

    def irrelevant(self):
        severity = self.mapped_severity()
        threshold = int(config.get('events', 'severity_threshold'))
        cutoff = self.cut_off_date()

        # Check severity threshold
        if severity < threshold:
            log.debug("Event skipped: severity %d below threshold %d (offset: %s)",
                      severity, threshold, self.offset)
            return True

        # Check cut-off date
        if self.creation_time < cutoff:
            # Format datetime objects directly
            event_date = self.creation_time.strftime('%Y-%m-%d %H:%M:%S')
            cutoff_date = cutoff.strftime('%Y-%m-%d %H:%M:%S')

            log.debug("Event skipped: creation time %s before cut-off date %s (offset: %s)",
                      event_date, cutoff_date, self.offset)
            return True

        return False

    def mapped_severity(self):
        """Map CrowdStrike severity to internal 1-5 scale"""
        severity_name = self['event'].get('SeverityName', 'Critical')
        name_to_value = {
            "Informational": 1,
            "Low": 2,
            "Medium": 3,
            "High": 4,
            "Critical": 5
        }
        return name_to_value.get(severity_name, 5)  # Default to 5 (highest) if unknown

    @property
    def event_type(self):
        return self['metadata']['eventType']

    @property
    def offset(self):
        return self['metadata']['offset']

    @property
    def uid(self):
        return str(self.feed_id) + '_' + str(self.offset)

    @property
    def severity(self):
        return self['event'].get('Severity', 5)

    @property
    def creation_time(self):
        return self.parse_cs_time(self['metadata']['eventCreationTime'])

    @property
    def sensor_id(self):
        # return self['event']['SensorId']
        return self['event'].get('SensorId') or self['event'].get('AgentId')

    @property
    def computer_name(self):
        return self['event'].get('ComputerName') or self['event'].get('Hostname')

    @classmethod
    def parse_cs_time(cls, cs_timestamp):
        return datetime.datetime.utcfromtimestamp(float(cs_timestamp) / 1000.0)

    @classmethod
    def cut_off_date(cls):
        return datetime.datetime.now() - datetime.timedelta(days=int(config.get('events', 'older_than_days_threshold')))


class Stream(dict):
    @property
    def token(self):
        return self['sessionToken']['token']

    @property
    def url(self):
        return self['dataFeedURL']

    @property
    def refresh_interval(self):
        return self['refreshActiveSessionInterval']

    @property
    def partition(self):
        match = re.match(r'.*\/sensors\/entities\/datafeed-actions/v1/([0-9a-zA-Z]+)\?',
                         self['refreshActiveSessionURL'])
        if not match or not match.group(1):
            raise Exception('Cannot parse stream partition from stream data: {}'.format(self))
        return match.group(1)

    @property
    def feed_id(self):
        if 'feed_id' not in self:
            match = re.match(r'.*\/sensors\/entities\/datafeed/v1/([0-9a-zA-Z]+)\?',
                             self['dataFeedURL'])
            if not match or not match.group(1):
                raise Exception('Cannot parse Feed ID from stream data: {}'.format(self))
            self['feed_id'] = match.group(1)
        return self['feed_id']
